@@ -19,7 +19,7 @@
 #' \code{'contrastReg'}, \code{'randomForest'}, \code{'gam'}.
 #' @param higher.y A logical value indicating whether higher (\code{TRUE}) or
 #' lower (\code{FALSE}) values of the outcome are more desirable. Default is \code{TRUE}.
-#' @param prop.cutoff A vector of numerical values (in (0, 1]) specifying percentiles of
+#' @param prop.cutoff A vector of numerical values (in `(0, 1]`) specifying percentiles of
 #' the estimated log CATE scores to define nested subgroups. Each element represents the
 #' cutoff to separate observations in nested subgroups (below vs above cutoff).
 #' The length of \code{prop.cutoff} is the number of nested subgroups.
@@ -29,9 +29,9 @@
 #' Allowed values include one of: \code{'glm'} for logistic regression with main effects only
 #' (default), or \code{'lasso'} for a logistic regression with main effects and LASSO penalization
 #' on two-way interactions (added to the model if not specified in \code{ps.model}).
-#' @param minPS A numerical value (in [0, 1]) below which estimated propensity scores should be
+#' @param minPS A numerical value (in `[0, 1]`) below which estimated propensity scores should be
 #' truncated. Default is \code{0.01}.
-#' @param maxPS A numerical value (in (0, 1]) above which estimated propensity scores should be
+#' @param maxPS A numerical value (in `(0, 1]`) above which estimated propensity scores should be
 #' truncated. Must be strictly greater than \code{minPS}. Default is \code{0.99}.
 #' @param initial.predictor.method A character vector for the method used to get initial outcome
 #' predictions conditional on the covariates in \code{init.model} in
@@ -81,67 +81,67 @@
 #'
 #' @return Returns a list containing the following components:
 #' \itemize{
-#'  \item{\code{ate.gaussian}: }{A vector of numerical values of length \code{prop.cutoff}
+#'  \item{\code{ate.gaussian}: }A vector of numerical values of length \code{prop.cutoff}
 #'  containing the estimated ATE in nested subgroups (defined by \code{prop.cutoff})
 #'  constructed based on the estimated CATE scores with Poisson regression.
-#'  Only provided if \code{score.method} includes \code{'gaussian'}.}
-#'  \item{\code{ate.boosting}: }{Same as \code{ate.gaussian}, but with the nested subgroups based
+#'  Only provided if \code{score.method} includes \code{'gaussian'}.
+#'  \item{\code{ate.boosting}: }Same as \code{ate.gaussian}, but with the nested subgroups based
 #'  the estimated CATE scores with boosting. Only provided if \code{score.method}
-#'  includes \code{'boosting'}.}
-#'  \item{\code{ate.twoReg}: }{Same as \code{ate.gaussian}, but with the nested subgroups based
+#'  includes \code{'boosting'}.
+#'  \item{\code{ate.twoReg}: }Same as \code{ate.gaussian}, but with the nested subgroups based
 #'  the estimated CATE scores with two regressions.
-#'  Only provided if \code{score.method} includes \code{'twoReg'}.}
-#'  \item{\code{ate.contrastReg}: }{Same as \code{ate.gaussian}, but with the nested subgroups based
+#'  Only provided if \code{score.method} includes \code{'twoReg'}.
+#'  \item{\code{ate.contrastReg}: }Same as \code{ate.gaussian}, but with the nested subgroups based
 #'  the estimated CATE scores with contrast regression.
-#'  Only provided if \code{score.method} includes \code{'contrastReg'}.}
-#'  \item{\code{ate.randomForest}: }{Same as \code{ate.gaussian}, but with the nested subgroups based
+#'  Only provided if \code{score.method} includes \code{'contrastReg'}.
+#'  \item{\code{ate.randomForest}: }Same as \code{ate.gaussian}, but with the nested subgroups based
 #'  the estimated CATE scores with random forest.
-#'  Only provided if \code{score.method} includes \code{'gam'}.}
-#'  \item{\code{ate.gam}: }{Same as \code{ate.gaussian}, but with the nested subgroups based
+#'  Only provided if \code{score.method} includes \code{'gam'}.
+#'  \item{\code{ate.gam}: }Same as \code{ate.gaussian}, but with the nested subgroups based
 #'  the estimated CATE scores with generalized additive model.
-#'  Only provided if \code{score.method} includes \code{'gam'}.}
-#'  \item{\code{score.gaussian}: }{A vector of numerical values of length n
+#'  Only provided if \code{score.method} includes \code{'gam'}.
+#'  \item{\code{score.gaussian}: }A vector of numerical values of length n
 #'  (number of observations in \code{data}) containing the estimated CATE scores
 #'  according to the linear regression. Only provided if \code{score.method}
-#'  includes \code{'gaussian'}.}
-#'  \item{\code{score.boosting}: }{Same as \code{score.gaussian}, but with estimated CATE score
+#'  includes \code{'gaussian'}.
+#'  \item{\code{score.boosting}: }Same as \code{score.gaussian}, but with estimated CATE score
 #'  according to boosting. Only provided if \code{score.method} includes
-#'  \code{'boosting'}.}
-#'  \item{\code{score.twoReg}: }{Same as \code{score.gaussian}, but with estimated CATE score
+#'  \code{'boosting'}.
+#'  \item{\code{score.twoReg}: }Same as \code{score.gaussian}, but with estimated CATE score
 #'  according to two regressions. Only provided if \code{score.method} includes
-#'  \code{'twoReg'}.}
-#'  \item{\code{score.contrastReg}: }{Same as \code{score.gaussian}, but with estimated CATE score
+#'  \code{'twoReg'}.
+#'  \item{\code{score.contrastReg}: }Same as \code{score.gaussian}, but with estimated CATE score
 #'  according to contrast regression. Only provided if \code{score.method} includes
-#'  \code{'contrastReg'}.}
-#'  \item{\code{score.randomForest}: }{Same as \code{score.gaussian}, but with estimated CATE score
+#'  \code{'contrastReg'}.
+#'  \item{\code{score.randomForest}: }Same as \code{score.gaussian}, but with estimated CATE score
 #'  according to random forest. Only provided if \code{score.method}
-#'  includes \code{'randomForest'}.}
-#'  \item{\code{score.gam}: }{Same as \code{score.gaussian}, but with estimated CATE score
+#'  includes \code{'randomForest'}.
+#'  \item{\code{score.gam}: }Same as \code{score.gaussian}, but with estimated CATE score
 #'  according to generalized additive model. Only provided if \code{score.method}
-#'  includes \code{'gam'}.}
-#'  \item{\code{fit}: }{Additional details on model fitting if \code{score.method}
-#'  includes 'boosting' or 'contrastReg':}
+#'  includes \code{'gam'}.
+#'  \item{\code{fit}: }Additional details on model fitting if \code{score.method}
+#'  includes 'boosting' or 'contrastReg':
 #'  \itemize{
-#'    \item{\code{result.boosting}: }{Details on the boosting model fitted to observations
+#'    \item{\code{result.boosting}: }Details on the boosting model fitted to observations
 #'    with treatment = 0 \code{($fit0.boosting)} and to observations with treatment = 1 \code{($fit1.boosting)}.
-#'    Only provided if \code{score.method} includes \code{'boosting'}.}
-#'    \item{\code{result.randomForest}: }{Details on the boosting model fitted to observations
+#'    Only provided if \code{score.method} includes \code{'boosting'}.
+#'    \item{\code{result.randomForest}: }Details on the boosting model fitted to observations
 #'    with treatment = 0 \code{($fit0.randomForest)} and to observations with treatment = 1 \code{($fit1.randomForest)}.
-#'    Only provided if \code{score.method} includes \code{'randomForest'}.}
-#'    \item{\code{result.gam}: }{Details on the boosting model fitted to observations
+#'    Only provided if \code{score.method} includes \code{'randomForest'}.
+#'    \item{\code{result.gam}: }Details on the boosting model fitted to observations
 #'    with treatment = 0 \code{($fit0.gam)} and to observations with treatment = 1 \code{($fit1.gam)}.
-#'    Only provided if \code{score.method} includes \code{'gam'}.}
-#'    \item{\code{result.contrastReg$sigma.contrastReg}: }{Variance-covariance matrix of
+#'    Only provided if \code{score.method} includes \code{'gam'}.
+#'    \item{\code{result.contrastReg$sigma.contrastReg}: }Variance-covariance matrix of
 #'    the estimated CATE coefficients in contrast regression.
-#'    Only provided if \code{score.method} includes \code{'contrastReg'}.}
+#'    Only provided if \code{score.method} includes \code{'contrastReg'}.
 #'  }
-#'  \item{\code{coefficients}: }{A data frame with the coefficients of the estimated CATE
+#'  \item{\code{coefficients}: }A data frame with the coefficients of the estimated CATE
 #'  score by \code{score.method}. The data frame has number of rows equal to the number of
 #'  covariates in \code{cate.model} and number of columns equal to \code{length(score.method)}.
 #'  If \code{score.method} includes \code{'contrastReg'}, the data frame has an additional
 #'  column containing the standard errors of the coefficients estimated with contrast regression.
 #'  \code{'boosting'}, \code{'randomForest'}, \code{'gam'} do not have coefficient results because these methods do not
-#'  express the CATE as a linear combination of coefficients and covariates.}
+#'  express the CATE as a linear combination of coefficients and covariates.
 #' }
 #'
 #' @details The CATE score represents an individual-level treatment effect, estimated with
@@ -167,8 +167,7 @@
 #'
 #' @references Yadlowsky, S., Pellegrini, F., Lionetto, F., Braune, S., & Tian, L. (2020).
 #' \emph{Estimation and validation of ratio-based conditional average treatment effects using
-#' observational data. Journal of the American Statistical Association, 1-18.}
-#' \url{https://www.tandfonline.com/doi/full/10.1080/01621459.2020.1772080}
+#' observational data. Journal of the American Statistical Association, 1-18.} DOI: 10.1080/01621459.2020.1772080.
 #'
 #' @seealso \code{\link{catecvmean}()} function
 #'
@@ -445,7 +444,7 @@ twoarmglmmean.dr  <- function(y, x.cate, trt, ps, f1.predictor, f0.predictor){
 #' \code{'lasso'} for a logistic regression with main effects and LASSO penalization on
 #' two-way interactions (added to the model if interactions are not specified in \code{ps.model}).
 #' Relevant only when \code{ps.model} has more than one variable.
-#' @param minPS A numerical value (in [0, 1]) below which estimated propensity scores should be
+#' @param minPS A numerical value (in `[0, 1]`) below which estimated propensity scores should be
 #' truncated. Default is \code{0.01}.
 #' @param maxPS A number above which estimated propensity scores should be trimmed; scalar
 #' @param initial.predictor.method A character vector for the method used to get initial
@@ -504,27 +503,25 @@ intxmean <- function(y, trt, x.cate, x.init, x.ps,
   result <- vector("list", length(score.method) + 1)
   names(result) <- c(paste0("result.", score.method), "best.iter")
 
-  N1 <- sum(trt)
-  N0 <- sum(1 - trt)
-  N <- N1 + N0
+  # Calculate the total number of patients who received the treatment (N1)
+  # and the total number of patients who did not receive the treatment (N0)
+  N1 <- sum(trt)               # N1: number of treated patients (trt = 1)
+  N0 <- length(trt) - N1        # N0: number of untreated patients (trt = 0)
+
+  # Total number of patients
+  N <- length(trt)              # N is simply the total number of patients
+
+  # Calculate p.aug: the number of covariates in x.cate plus 1 for the intercept
   p.aug <- ncol(x.cate) + 1
 
-  #datatot <- data.frame(y, x.cate, time)
-  #colnames(datatot) <- c("y", colnames(x.cate), "time")
+  datatot <- data.frame("y" = y, x.cate)
+  datatot.init <- data.frame("y" = y, x.init)
 
-  datatot <- data.frame(y, x.cate)
-  colnames(datatot) <- c("y", colnames(x.cate))
+  ######### cross-fitting
 
-  datatot.init <- data.frame(y, x.init)
-  colnames(datatot.init) <- c("y", colnames(x.init))
-
-  ######### cross-fitting  ---------------------------------------------------------------
-
-  index1 <- rep(1:Kfold, floor(N1 / Kfold))
-  if (N1 > Kfold * floor(N1 / Kfold)) index1 <- c(index1, 1:(N1 - Kfold * floor(N1 / Kfold)))
-
-  index0 <- rep(1:Kfold, floor(N0 / Kfold))
-  if (N0 > Kfold * floor(N0 / Kfold)) index0 <- c(index0, Kfold + 1 - 1:(N0 - Kfold * floor(N0 / Kfold)))
+  # Generate indices for N1 (normal order) and N0 (reverse order)
+  index1 <- generate_kfold_indices(N1, Kfold, reverse = FALSE)
+  index0 <- generate_kfold_indices(N0, Kfold, reverse = TRUE)
 
   delta.twoReg.mat <- delta.contrastReg.mat <- matrix(NA, B, p.aug)
   sigma.contrastReg.mat <- matrix(0, p.aug, p.aug)
